# This script parses the XML weather data from http://www.wunderground.com
# 
# The API is documented at: http://wiki.wunderground.com/index.php/API_-_XML
#
# The actual API is hosted at: http://api.wunderground.com
#

package WUNDERGROUNDParser;
use strict;
use POSIX;
use XML::Simple;

our $VERSION = 0.1;

sub getIcon {
    my $condition = shift;
    my $icon;

    if ( ($condition =~ /snow/i) || ($condition =~ /flurries/i)) {
        $icon = 'flurries.png';
        $icon = 'rainsnow.png' if ($condition =~ /rain/i);
        $icon = 'snowshow.png' if ($condition =~ /heavy/i);
    }
    elsif ($condition =~ /fog/i) {
        $icon = 'fog.png';
    }
    elsif ( ($condition =~ /rain/i) || ($condition =~ /showers/i) || ($condition =~ /tstorm/i) ) {
        $icon = 'showers.png';
        $icon = 'lshowers.png'  if ( ($condition =~ /chance/i) ||
                                     ($condition =~ /few/i));
        $icon = 'rainsnow.png'  if ( ($condition =~ /snow/i)   ||
                                     ($condition =~ /flurries/i));
        $icon = 'thunshowers.png' if ( ($condition =~ /thunder/i) ||
                                       ($condition =~ /tstorm/i));;
    }
    elsif ($condition =~ /cloud/i) {
        $icon = 'cloudy.png';
        $icon = 'mcloudy.png' if ($condition =~ /mostly/i);
        $icon = 'pcloudy.png' if ( ($condition =~ /few/i)    ||
                                   ($condition =~ /mix/i)    ||
                                   ($condition =~ /partly/i) ||
                                   ($condition =~ /period/i) );
    }
    elsif ($condition =~ /clear/i) {
        $icon = 'fair.png';
    }
    elsif ($condition =~ /sun/i) {
        $icon = 'sunny.png';
    }
    else {
        $icon = 'unknown.png';
    }

    return $icon;
}

sub doParseGeo {
    my ($data) = @_;

    my $xml = XMLin($data, forcearray => 1);
    die if (!$xml);

    my @results;
    my $i = 0;

    if ($xml->{location}) {
        # Multiple hits
        foreach my $item (@{$xml->{location}}) {
            $results[$i++] = $item->{name}->[0];
        }

        my %saw;
        @saw{@results} = ();
        @results = sort keys %saw;  # remove sort if undesired
    } else {
        # Single result
        my $location = $xml->{city}->[0];

        if ($xml->{type} =~ /INTLCITY/) {
            $location = $location . ', ' . $xml->{country}->[0];
        } elsif ($xml->{type} =~ /CITY/) {
            $location = $location . ', ' . $xml->{state}->[0];
        }

        push(@results, $location);
    }

    return @results;
}

sub doParseCurrent {
    my ($data, $units, %results) = @_;

    my $temp_suffix = '_c';
    my $pres_suffix = '_mb';
    my $dist_suffix = '_km';
    if ($units =~ /ENG/) {
        $temp_suffix = '_f';
        $pres_suffix = '_in';
        $dist_suffix = '_mi';
    }

    my $xml = XMLin($data);
    die if (!$xml);

    $results{'cclocation'} = $xml->{display_location}->{full};
    $results{'3dlocation'} = $results{'cclocation'};
    $results{'6dlocation'} = $results{'cclocation'};

    $results{'observation_time'} = $xml->{observation_time};
    $results{'observation_time_rfc822'} = $xml->{observation_time_rfc822};

    $results{'weather'} = $xml->{weather};
    $results{'weather_icon'} = getIcon($xml->{icon});

    $results{'relative_humidity'} = $xml->{relative_humidity};
    $results{'wind_dir'} = $xml->{wind_dir};
    $results{'wind_degrees'} = $xml->{wind_degrees};
        
    $results{'temp'} = $xml->{'temp' . $temp_suffix};
    $results{'dewpoint'} = $xml->{'dewpoint' . $temp_suffix};
    $results{'heat_index'} = $xml->{'heat_index' . $temp_suffix};
    $results{'windchill'} = $xml->{'windchill' . $temp_suffix};
    $results{'appt'} = $results{'heat_index'} unless $results{'heat_index' =~ /NA/};
    $results{'appt'} = $results{'windchill'} unless $results{'windchill' =~ /NA/};
    
    $results{'pressure'} = $xml->{'pressure' . $pres_suffix};
    $results{'visibility'} = $xml->{'visibility' . $dist_suffix};

    $results{'wind_speed'} = $xml->{wind_mph};
    $results{'wind_spdgst'} = $xml->{wind_gust_mph} unless ref($xml->{wind_gust_mph}) eq 'HASH';

    if ($units =~ /SI/) {
        $results{'wind_speed'} *= 1.6;
        $results{'wind_spdgst'} *= 1.6 if length $results{'wind_spdgst'} > 0;
    }

    return %results;
}

sub doParseForecast {
    my ($data, $units, %results) = @_;

    my $xml = XMLin($data);
    die if (!$xml);

    $results{'updatetime'} = $xml->{txt_forecast}->{date}
                    unless ref($xml->{txt_forecast}->{date}) eq 'HASH';

    my $i = 0; 
    foreach my $item (@{$xml->{simpleforecast}->{forecastday}}) {
        $i = $item->{period} - 1;
        $results{"date-$i"} = $item->{date}->{monthname} . ' ' . 
                            $item->{date}->{day} . ', ' . $item->{date}->{year};

        $results{"icon-$i"} = getIcon($item->{icon});
        if ($units =~ /SI/) {
            $results{"low-$i"}  = $item->{low}->{celsius};
            $results{"high-$i"} = $item->{high}->{celsius};
        } else {
            $results{"low-$i"}  = $item->{low}->{fahrenheit};
            $results{"high-$i"} = $item->{high}->{fahrenheit};
        }
    }

    return %results;
}

1;
# vim: set expandtab tabstop=4 shiftwidth=4:
